;
; Function to get a sequence variable from dods and return it as a
; struct.  This function works by allocating chunks of
; memory as needed, and copying the data into these arrays.  This
; avoids the overhead of creating an array one element larger for each
; new row.
;
;    var_p - (Input )
;       Pointer to DDS variable.
;    var - (Output)
;       Returns the DDS variable's data.
;    name - (Output)
;       Returns the DDS variable's name.
;
;    TODO: If the code for all the primative types in dods_var were
;    copied in here and modified slightly, it would cut the number of
;    calls to dods_var_info in half, saving a few seconds on large datasets

FUNCTION DODS_VAR_SEQ, conn, dds, var_p, var, name, seq, blocksize
    
   ; print, "DODS_VAR_SEQ"
   DODS_VAR_INFO, var_p, type, var_name, pix
    
   CASE type OF
      !dods_sequence_c: BEGIN
         
         ; This was moved outside and to the bottom of the while loop to fix a 
         ; bug where the last entry was repeated.  06/12/00 rph.
            
         more_seq = DODS_SEQ(conn, dds, var_p)
         more_seq = 1           
         
         WHILE (more_seq EQ 1UL) DO BEGIN
                
            WHILE (pix NE 0UL) DO BEGIN
                    
               sub_var_p = 0UL
               DODS_NEXT, var_p, pix, sub_var_p
                    
               status = DODS_VAR_SEQ(conn, dds, sub_var_p, curr_var, $
                                     curr_name, seq, blocksize) 
               
               IF ((status GT 0UL) AND (N_ELEMENTS(curr_var) GT 0UL)) THEN BEGIN 
                        ; If there are no elements in the structure, 
                        ; or we haven't gone all the way through the 
                        ; elements yet, set tag_num to -1
                                                                  
                  IF ( N_ELEMENTS(seq) GE 1 ) THEN BEGIN
                     IF ( seq.index GT 0 ) THEN BEGIN
                                ;DODS_FIX_NAME, curr_name

                                ; print, TAG_NAMES(seq), ' - ', curr_name
                                ;tag_num = (WHERE(TAG_NAMES(seq) EQ curr_name))[0]
                        tag_num = seq.tag
                                ;print, 'tag_num: ', tag_num
                     ENDIF ELSE tag_num = -1
                                ; print, tag_num, ' - ', seq.tag
                  ENDIF ELSE tag_num = -1
                  
                        
                        ; This means that the structure isn't complete
                  IF (tag_num LT 0L) THEN BEGIN
                     DODS_FIX_NAME, curr_name
                     
                     tags = N_TAGS(seq)
                     IF ( tags GE 1) THEN BEGIN 
                        seq = CREATE_STRUCT(TEMPORARY(seq), curr_name, $
                                            PTR_NEW(INDGEN(blocksize, TYPE=SIZE(curr_var, /TYPE))))
                        (*seq.(tags))[0] = TEMPORARY(curr_var)
                        seq.columns = seq.columns + 1
                        seq.tag = tags + 1
                        
                     ENDIF ELSE BEGIN 
                        seq = CREATE_STRUCT("BLOCK_NUM", 1UL, "INDEX", 0UL, $
                                            "TAG", 4UL, "COLUMNS", 1UL, $
                                            curr_name, $
                                            PTR_NEW(INDGEN(blocksize, TYPE=size(curr_var, /TYPE))))
                        (*seq.(4))[0] = TEMPORARY(curr_var)
                     ENDELSE
                     
                  ENDIF ELSE BEGIN ; Concatenate existing data
                     
                     ; If we have room, set the array value, otherwise
                     ; allocate more elements
                     IF ( seq.index LT blocksize * seq.block_num) THEN BEGIN 
                       ; print, curr_var
                        (*seq.(tag_num))[seq.index] = TEMPORARY(curr_var)
                     ENDIF ELSE BEGIN
                        *seq.(tag_num) = [ TEMPORARY(*seq.(tag_num)), $ 
                                           INDGEN(blocksize, $
                                                  TYPE=size(curr_var, /TYPE)) ]
                        (*seq.(tag_num))[seq.index] = TEMPORARY(curr_var)
                     ENDELSE
                     
                     seq.tag = seq.tag + 1
                  ENDELSE
                  
                  IF ( seq.tag - 4 GE seq.columns ) THEN BEGIN
                     seq.tag = 4
                  ENDIF 
                  
               ENDIF ELSE pix = 0UL
            ENDWHILE
            
                
                ; deserialize more of the sequence, and while we're at it
                ; indicate that it's moved on to the next row.  If 
                ; the index goes past the boundary of a block, all
                ; the arrays should be enlarged, so just increment.
                
            more_seq = DODS_SEQ(conn, dds, var_p)
            
            IF(more_seq EQ 1) THEN BEGIN
               IF ( seq.index GE blocksize * seq.block_num ) THEN BEGIN
                  seq.block_num = seq.block_num + 1
               ENDIF
               seq.index = seq.index + 1
            ENDIF
            
            DODS_VAR_INFO, var_p, type, var_name, pix
         ENDWHILE
         
         RETURN, 0UL
      END
      ELSE: BEGIN
         RETURN, DODS_VAR(conn, dds, var_p, var, name)
      END
   ENDCASE
END



